@@ -307,3 +307,135 @@ GetCors <- function(mat.list, ws, num.sets){
   }
   return(cors)
 }
+
+# FIt-SNE helper function for calling fast_tsne from R
+#
+# Based on Kluger Lab code on https://github.com/ChristophH/FIt-SNE
+# commit ec25f1b36598a2d21869d10a258ac366a12f0b05
+#
+#' @importFrom utils file_test
+#
+fftRtsne <- function(
+  X,
+  dims = 2,
+  perplexity = 30,
+  theta = 0.5,
+  check_duplicates = TRUE,
+  max_iter = 1000,
+  fft_not_bh = TRUE,
+  ann_not_vptree = TRUE,
+  stop_lying_iter = 250,
+  exaggeration_factor = 12.0,
+  no_momentum_during_exag = FALSE,
+  start_late_exag_iter = -1.0,
+  late_exag_coeff = 1.0,
+  n_trees = 50,
+  search_k = -1,
+  rand_seed = -1,
+  nterms = 3,
+  intervals_per_integer = 1,
+  min_num_intervals = 50,
+  data_path = NULL,
+  result_path = NULL,
+  fast_tsne_path = NULL,
+  nthreads = getOption('mc.cores', default = 1),
+  ...
+) {
+  if (is.null(x = data_path)) {
+    data_path <- tempfile(pattern = 'fftRtsne_data_', fileext = '.dat')
+  }
+  if (is.null(x = result_path)) {
+    result_path <- tempfile(pattern = 'fftRtsne_result_', fileext = '.dat')
+  }
+  if (is.null(x = fast_tsne_path)) {
+    fast_tsne_path <- system2(command = 'which', args = 'fast_tsne', stdout = TRUE)
+    if (length(x = fast_tsne_path) == 0) {
+      stop("no fast_tsne_path specified and fast_tsne binary is not in the search path")
+    }
+  }
+  fast_tsne_path <- normalizePath(path = fast_tsne_path)
+  if (!file_test(op = '-x', x = fast_tsne_path)) {
+    stop("fast_tsne_path '", fast_tsne_path, "' does not exist or is not executable")
+  }
+  is.wholenumber <- function(x, tol = .Machine$double.eps ^ 0.5) {
+    return(abs(x = x - round(x = x)) < tol)
+  }
+  if (!is.numeric(x = theta) || (theta < 0.0) || (theta > 1.0) ) {
+    stop("Incorrect theta.")
+  }
+  if (nrow(x = X) - 1 < 3 * perplexity) {
+    stop("Perplexity is too large.")
+  }
+  if (!is.matrix(x = X)) {
+    stop("Input X is not a matrix")
+  }
+  if (!(max_iter > 0)) {
+    stop("Incorrect number of iterations.")
+  }
+  if (!is.wholenumber(x = stop_lying_iter) || stop_lying_iter < 0) {
+    stop("stop_lying_iter should be a positive integer")
+  }
+  if (!is.numeric(x = exaggeration_factor)) {
+    stop("exaggeration_factor should be numeric")
+  }
+  if (!is.wholenumber(x = dims) || dims <= 0) {
+    stop("Incorrect dimensionality.")
+  }
+  if (search_k == -1) {
+    search_k = n_trees * perplexity * 3
+  }
+  # if (fft_not_bh) {
+  #   nbody_algo <- 2
+  # } else {
+  #   nbody_algo <- 1
+  # }
+  nbody_algo <- ifelse(test = fft_not_bh, yes = 2, no = 1)
+  # if (ann_not_vptree) {
+  #   knn_algo <- 1
+  # }else{
+  #   knn_algo <- 2
+  # }
+  knn_algo <- ifelse(test = ann_not_vptree, yes = 1, no = 2)
+  tX = c(t(x = X))
+  f <- file(data_path, "wb")
+  n = nrow(x = X)
+  D = ncol(x = X)
+  writeBin(object = as.integer(x = n), con = f, size = 4)
+  writeBin(object = as.integer(x = D), con = f, size = 4)
+  writeBin(object = as.numeric(x = 0.5), con = f, size = 8) #theta
+  writeBin(object = as.numeric(x = perplexity), con = f, size = 8) #theta
+  writeBin(object = as.integer(x = dims), con = f, size = 4) #theta
+  writeBin(object = as.integer(x = max_iter), con = f, size = 4)
+  writeBin(object = as.integer(x = stop_lying_iter), con = f, size = 4)
+  writeBin(object = as.integer(x = -1), con = f, size = 4) #K
+  writeBin(object = as.numeric(x = -30.0), con = f, size = 8) #sigma
+  writeBin(object = as.integer(x = nbody_algo), con = f, size = 4)  #not barnes hut
+  writeBin(object = as.integer(x = knn_algo), con = f, size = 4)
+  writeBin(object = as.numeric(x = exaggeration_factor), con = f, size = 8) #compexag
+  writeBin(object = as.integer(x = no_momentum_during_exag), con = f, size = 4)
+  writeBin(object = as.integer(x = n_trees), con = f, size = 4)
+  writeBin(object = as.integer(x = search_k), con = f, size = 4)
+  writeBin(object = as.integer(x = start_late_exag_iter), con = f, size = 4)
+  writeBin(object = as.numeric(x = late_exag_coeff), con = f, size = 8)
+  writeBin(object = as.integer(x = nterms), con =  f, size = 4)
+  writeBin(object = as.numeric(x = intervals_per_integer), con =  f, size = 8)
+  writeBin(object = as.integer(x = min_num_intervals), con =  f, size = 4)
+  tX = c(t(x = X))
+  writeBin(object = tX, con = f)
+  writeBin(object = as.integer(x = rand_seed), con = f, size = 4)
+  close(f)
+  flag <- system2(command = fast_tsne_path, args = c(data_path, result_path, nthreads))
+  if (flag != 0) {
+    stop('tsne call failed');
+  }
+  f <- file(description = result_path, open = "rb")
+  initialError <- readBin(f, integer(), n = 1, size = 8)
+  n <- readBin(con = f, what = integer(), n = 1, size = 4)
+  d <- readBin(con = f, what = integer(), n = 1, size = 4)
+  Y <- readBin(con = f, what = numeric(), n = n * d)
+  Yout <- t(x = matrix(data = Y, nrow = d))
+  close(f)
+  file.remove(data_path)
+  file.remove(result_path)
+  return(Yout)
+}
