@@ -157,9 +157,31 @@ SingleFeaturePlot <- function(
   min.cutoff,
   max.cutoff,
   no.axes,
+  no.title=FALSE,
   no.legend,
-  dark.theme
+  dark.theme,
+  vector.friendly=FALSE,
+  png.file=NULL,
+  png.arguments=c(10,10,100)
 ) {
+  
+  #first, consider vector friendly case
+  if (vector.friendly) {
+    #browser()
+    previous_call <- blank_call <- png_call <-  match.call()
+    blank_call$pt.size <- -1;  blank_call$vector.friendly=F;
+    png_call$no.axes=T; png_call$no.legend=T;  png_call$vector.friendly=F; png_call$no.title=TRUE;
+    blank_plot <- eval(blank_call, sys.frame(sys.parent()))
+    png_plot <- eval(png_call, sys.frame(sys.parent()))
+    png.file <- SetIfNull(png.file,"temp_png.png")
+    # browser()
+    ggsave(filename = png.file, plot = png_plot,width=png.arguments[1], height=png.arguments[2], dpi=png.arguments[3])
+    to_return=AugmentPlot(blank_plot,png.file)
+    return(to_return)
+  }
+  
+  
+  
   data.gene <- na.omit(object = data.frame(data.use[feature, ]))
   #   Check for quantiles
   min.cutoff <- SetQuantile(cutoff = min.cutoff, data = data.gene)
@@ -225,8 +247,11 @@ SingleFeaturePlot <- function(
       )
     }
   }
+  if (dark.theme) {
+    p <- p + DarkTheme()
+  }
   if (no.axes) {
-    p <- p + labs(title = feature, x ="", y="") + theme(
+    p <- p + theme(
       axis.line = element_blank(),
       axis.text.x = element_blank(),
       axis.text.y = element_blank(),
@@ -234,15 +259,16 @@ SingleFeaturePlot <- function(
       axis.title.x = element_blank(),
       axis.title.y = element_blank()
     )
+    if (!no.title) p <- p + labs(title = feature, x ="", y="")
+    if (no.title) p <- p + labs(x ="", y="")
   } else {
-    p <- p + labs(title = feature, x = dim.codes[1], y = dim.codes[2])
+    if (no.title) p <- p + labs(x = dim.codes[1], y = dim.codes[2])
+    if (!(no.title)) p <- p + labs(title = feature, x = dim.codes[1], y = dim.codes[2])
   }
   if (no.legend) {
     p <- p + theme(legend.position = 'none')
   }
-  if (dark.theme) {
-    p <- p + DarkTheme()
-  }
+
   return(p)
 }
 
@@ -640,32 +666,31 @@ SingleVlnPlot <- function(
       x = factor(x = ident),
       y = feature
     )
-  ) +
+  ) + 
     geom_violin(
       scale = "width",
       adjust = adjust.use,
       trim = TRUE,
       mapping = aes(fill = factor(x = ident))
     ) +
-    theme(
-      legend.position = legend.position,
-      axis.title.x = element_text(
-        face = "bold",
-        colour = "#990000",
-        size = size.x.use
-      ),
-      axis.title.y = element_text(
-        face = "bold",
-        colour = "#990000",
-        size = size.y.use
-      )
-    ) +
     guides(fill = guide_legend(title = NULL)) +
     geom_jitter(height = 0, size = point.size.use) +
     xlab("Identity") +
     NoGrid() +
     ggtitle(feature) +
-    theme(plot.title = element_text(size = size.title.use, face = "bold"))
+    theme(plot.title = element_text(size = size.title.use, face = "bold"),
+          legend.position = legend.position,
+          axis.title.x = element_text(
+              face = "bold",
+              colour = "#990000",
+              size = size.x.use
+          ),
+          axis.title.y = element_text(
+              face = "bold",
+              colour = "#990000",
+              size = size.y.use
+          )
+   )
   plot <- plot + ggtitle(feature.name)
   if (y.log) {
     plot <- plot + scale_y_log10()
@@ -685,10 +710,10 @@ SingleVlnPlot <- function(
     plot <- plot + scale_fill_manual(values = cols.use)
   }
   if (x.lab.rot) {
-    plot <- plot + theme(axis.text.x = element_text(angle = 45, hjust = 1))
+    plot <- plot + theme(axis.text.x = element_text(angle = 45, hjust = 1, size=size.x.use))
   }
   if (y.lab.rot) {
-    plot <- plot + theme(axis.text.x = element_text(angle = 90))
+    plot <- plot + theme(axis.text.x = element_text(angle = 90, size=size.y.use))
   }
   if (remove.legend) {
     plot <- plot + theme(legend.position = "none")
@@ -851,6 +876,27 @@ SetYAxisGG <- function(x = 16, y = "#990000", z = "bold", x2 = 12) {
   ))
 }
 
+# Get a gradient legend for palette of colours
+#
+# @param palette A palette of colours in hexadecimal form
+# @param group An index matching the palette
+# This allows to give a name for the palette 
+#
+# @return A grob object for the gradient legend
+#
+GetGradientLegend <- function(palette, group) {
+  # Plot for each palette a gradient legend
+  p <- ggplot(data = as.data.frame(palette), mapping = aes(x = NA, y = NA)) +
+    geom_point(mapping = aes(colour = 1:20)) +
+    scale_colour_gradientn(colours = palette, breaks = c(1, 20), label = c("Min", "Max"), name = group,
+                           guide = guide_colourbar(title.position = "top", title.hjust = 0.5, title.vjust = 0.5)) +
+    theme(legend.direction = "horizontal")
+  # Get legend from plot
+  l <- cowplot::get_legend(plot = p)
+  # Return legend grob
+  return(l)
+}
+
 #heatmap.2, but does not draw a key.
 #unclear if this is necessary, but valuable to have the function coded in for modifications
 #
