@@ -580,3 +580,20 @@ LengthCheck <- function(values, cutoff = 0) {
     FUN.VALUE = logical(1)
   ))
 }
+
+# Reverse the vector x and return the value at the Nth index. If N is larger
+# than the length of the vector, return the last value in the reversed vector.
+#
+# @param x vector of interest
+# @param N index in reversed vector
+#
+# @return returns element at given index
+#
+MaxN <- function(x, N = 2){
+  len <- length(x)
+  if(N > len) {
+    warning('N greater than length(x).  Setting N=length(x)')
+    N <- length(x)
+  }
+  sort(x, partial = len - N + 1)[len - N + 1]
+}
